@@ -40,7 +40,7 @@ from pycom.ext import checks
 _STRING_AND_NONE = string_types + (None.__class__,)
 
 
-@pycom.interface(pycom.constants.IFACE_NAMESERVER)
+@pycom.interface(pycom.constants.NS_INTERFACE)
 class NameServer(pycom.Service):
     """Class representing nameserver for PyCOM."""
 
