@@ -28,84 +28,31 @@
 
 import json
 
+import zerojson
+
 from . import constants
 
 
-class Request(object):
-    """Request object.
-
-    .. attribute:: interface
-
-       Current interface name
-
-    .. attribute:: method
-
-       Current method name
-
-    .. attribute:: session
-
-       Session for current component - dict-like object of
-       type :class:`pycom.base.BaseSession` (or subclass)
-
-    .. attribute:: args
-
-       Arguments passed by client side as Python object
-
-    """
-
-    def __init__(self, interface, method, session, args):
-        """Initializer."""
-        self.interface = interface
-        self.method = method
-        self.session = session if session is not None else Session(None)
-        self.args = args
-
-    def response(self, result):
-        """Build response object to return from method.
-
-        *result* is a JSON-serializable entity to return to client.
-
-        Returns :class:`pycom.Response` object.
-
-        """
-        return Response(self, result=result)
-
-
-class Response(object):
-    """Response object.
-
-    .. attribute:: request
-
-       :class:`pycom.Request` object that issued this response
-
-    .. attribute:: result
-
-       Call result as Python object
-
-    """
-
-    def __init__(self, request, result=None, code=0, message=None,
-            session_id=None):
-        """Initializer."""
-        self.request = request
-        self.result = result
-        self.code = code
-        self.message = message
-        self.session_id = session_id or (
-            request and request.session.session_id)
-
-
 class BaseComponent(object):
     """Abstract base class for components.
 
     Any component can be used as a context manager.
 
+    .. attribute:: name
+
+       Interface name.
+
     """
 
-    __slots__ = ()
+    __slots__ = ("name", "context")
+
+    def __init__(self, interface, context=None):
+        """Constructor."""
+        self.name = interface
+        self.context = context
 
     def invoke(self, method_name, args=None,
-            timeout=None):  # pylint: disable-msg=W0613
+            timeout=None):
         """Calls *method_name* on a service, passing *args* as argument.
 
         *args* should be anything JSON-serializable.
@@ -118,6 +65,29 @@ class BaseComponent(object):
 
         Returns call result as a JSON-serializable entity.
 
+        """
+        request = zerojson.Request(self.name, method_name, args=args)
+
+        if self.context is not None:
+            for hook in self.context.prehooks:
+                request = hook(self, request)
+
+        response = self.invoke_with_request(request, timeout=timeout)
+
+        if self.context is not None:
+            for hook in self.context.posthooks:
+                response = hook(self, response)
+
+        return response.result
+
+    def invoke_with_request(self, request,
+            timeout=None):  # pylint: disable-msg=W0613
+        """Invoke method using given request object.
+
+        Must be overriden in a real component class.
+
+        Returns :class:`zerojson.Response` object.
+
         """
         raise NotImplementedError()  # pragma: no cover
 
@@ -138,6 +108,10 @@ class BaseComponent(object):
     __exit__ = lambda self, *args: self.close()
 
 
+class ConfigurationError(Exception):
+    """Error found in configuration."""
+
+
 #: Read-only dictionary with current configuration
 configuration = {}
 
@@ -161,28 +135,6 @@ def configure(*args, **kwargs):
     configuration.update(kwargs)
 
 
-class Session(dict):
-    """Class representing session.
-
-    Inherits from `dict`, provides one additional attribute:
-
-    .. attribute: session_id
-
-       UUID of session
-
-    """
-
-    def __init__(self, session_id, *args, **kw):
-        """Constructor."""
-        self._owner = kw.pop("session_factory", None)
-        super(Session, self).__init__(*args, **kw)
-        self.session_id = session_id
-
-    def discard(self):
-        """Discard this sessions and all it's data."""
-        if self._owner is not None:
-            self._owner.drop(self.session_id)
-
 # Private
 
 
