@@ -26,7 +26,7 @@
 
 """Helpers to ease creating services."""
 
-from six import iteritems
+from six import iteritems, string_types
 
 from . import constants, interfaces
 
@@ -52,15 +52,16 @@ def interface(name):
     return _wrapper
 
 
-def method(name, body=None, method_factory=None, results=()):
+def method(name=None, body=None, method_factory=None, results=()):
     """Decorator for method *name*.
 
     Method should take at least one positional argument: *request*,
-    which is an object of type :class:`pycom.Request`.
+    which is an object of type :class:`zerojson.Request`.
 
     Method should return either JSON-serializable entity,
     or result of `request.response()` call
-    (see :meth:`pycom.Request.response`), i.e. :class:`pycom.Response` object.
+    (see :meth:`zerojson.Request.response`),
+    i.e. :class:`zerojson.Response` object.
 
     Example::
 
@@ -69,8 +70,8 @@ def method(name, body=None, method_factory=None, results=()):
         @pycom.interface("com.example.interface.foo")
         class ClassProvidingInterface(pycom.Service):
 
-            @pycom.method("method_name")
-            def my_method(self, request): print request.args
+            @pycom.method
+            def method_name(self, request): print request.args
 
     If *body* is present, this function does not act as a decorator,
     but directly return appropriate object for
@@ -98,8 +99,8 @@ def method(name, body=None, method_factory=None, results=()):
         @pycom.interface("com.example.interface.foo")
         class ClassProvidingInterface(pycom.Service):
 
-            @pycom.method("method_name")
-            def my_method(self, request, arg1, arg2, arg3=None):
+            @pycom.method
+            def method_name(self, request, arg1, arg2, arg3=None):
                 print arg1, arg2, arg3
 
     PyCOM will use Python's introspection features to guess that
@@ -116,17 +117,33 @@ def method(name, body=None, method_factory=None, results=()):
         @pycom.interface("com.example.interface.foo")
         class ClassProvidingInterface(pycom.Service):
 
-            @pycom.method("method_name", results=("res1", "res2"))
-            def my_method(self, request, arg1, arg2, arg3=None):
+            @pycom.method(results=("res1", "res2"))
+            def method_name(self, request, arg1, arg2, arg3=None):
                 return arg2, arg3
 
+    Finally, you can explicitly state remote method name::
+
+        import pycom
+
+        @pycom.interface("com.example.interface.foo")
+        class ClassProvidingInterface(pycom.Service):
+
+            @pycom.method("method_name")
+            def method_impl(self, request): print request.args
+
     """
     method_factory = method_factory or interfaces.Method
     if body is None:
-        def _wrapper(wrapped):  # pylint: disable-msg=C0111
+        if name is not None and not isinstance(name, string_types):
+            name, wrapped = name.__name__, name
             wrapped.__method__ = method_factory(wrapped, name, results)
             return wrapped
 
+        def _wrapper(wrapped):  # pylint: disable-msg=C0111
+            new_name = name if name is not None else wrapped.__name__
+            wrapped.__method__ = method_factory(wrapped, new_name, results)
+            return wrapped
+
         return _wrapper
     else:
         return method_factory(body, name)
@@ -147,8 +164,8 @@ class Service(object):
         class ClassProvidingInterface(pycom.Service):
             "My Demo Service"
 
-            @pycom.method("method_name", results=("res1", "res2"))
-            def my_method(self, request, arg1, arg2, arg3=None):
+            @pycom.method(results=("res1", "res2"))
+            def method_name(self, request, arg1, arg2, arg3=None):
                 "My Demo Method"
                 return arg2, arg3
 
