@@ -30,7 +30,9 @@ import inspect
 
 from six import itervalues
 
-from . import base, exceptions
+import zerojson
+
+from . import base
 
 
 registry = {}
@@ -49,10 +51,6 @@ class Interface(base.BaseComponent):
 
        Stored owner class instance.
 
-    .. attribute:: name
-
-       Interface name.
-
     .. attribute:: methods
 
        Dictionary of methods for this class, keys being method names,
@@ -60,18 +58,17 @@ class Interface(base.BaseComponent):
 
     """
 
-    __slots__ = ("instance", "name", "methods")
+    __slots__ = ("instance", "methods")
 
     def __init__(self, wrapped, name):
         """C-tor. Never call it directly, use pycom.interface decorator."""
-        super(Interface, self).__init__()
+        super(Interface, self).__init__(name)
 
         if name in registry and not isinstance(registry[name], wrapped):
             raise RuntimeError("Interface %s is already registered "
                 "for object %s" % (name, repr(registry[name])))
 
         self.instance = wrapped()
-        self.name = name
         self.methods = {}
 
         for key in dir(wrapped):
@@ -100,28 +97,19 @@ class Interface(base.BaseComponent):
 
         self.methods[method_info.name] = method_info
 
-    def invoke(self, method_name, args=None, **kw):  # pylint:disable-msg=W0613
-        """Invoke given method with args
-        (see :meth:`pycom.BaseComponent.invoke` for details).
-
-        *timeout* is ignored and provided only for
-        compatibility with other components.
-
-        """
-        return self.invoke_with_request(
-                base.Request(self.name, method_name, None, args)
-            ).result
-
-    def invoke_with_request(self, request):
+    def invoke_with_request(self, request,
+            timeout=None):
         """Invoke method using given request object.
 
-        Returns :class:`pycom.Response` object.
+        *timeout* is ignored here.
+
+        Returns :class:`zerojson.Response` object.
 
         """
         try:
             method_info = self.methods[request.method]
         except KeyError:
-            raise exceptions.MethodNotFound("Method '%s' not found in "
+            raise zerojson.MethodNotFound("Method '%s' not found in "
                 "interface %s" % (request.method, self.name))
 
         return method_info.call(self, request)
@@ -163,8 +151,8 @@ class Method(object):
            .. attribute:: data
               :noindex:
 
-              Data to be processed (:class:`pycom.Request` for prehooks,
-              :class:`pycom.Response` for posthooks).
+              Data to be processed (:class:`zerojson.Request` for prehooks,
+              :class:`zerojson.Response` for posthooks).
 
        Example from tests::
 
@@ -180,7 +168,7 @@ class Method(object):
         class InterfaceWithMethods(pycom.Service):
 
             @setup_my_hook
-            @pycom.method("...")
+            @pycom.method
             def method1(self, request):
                 pass  # ...
 
@@ -226,18 +214,18 @@ class Method(object):
         """Called once *owner_iface* is fully configured."""
 
     def call(self, iface, request):
-        """Call this method with given :class:`pycom.protocol.Request`.
+        """Call this method with given :class:`pycom.zerojson.Request`.
 
         *iface* should be :class:`pycom.interfaces.Interface` object.
 
-        Returns :class:`pycom.Response` object.
+        Returns :class:`zerojson.Response` object.
 
         """
         if self.required_arguments or self.optional_arguments:
             if request.args is None:
                 request.args = {}
             if not isinstance(request.args, dict):
-                raise exceptions.BadRequest("Expected dictionary as argument "
+                raise zerojson.BadRequest("Expected dictionary as argument "
                     "for %s.%s" % (iface.name, self.name))
 
         for hook in self.prehooks:
@@ -250,9 +238,19 @@ class Method(object):
             if name in request.args)
 
         response = self.wrapped(iface.instance, request, *args, **kwargs)
-        if not isinstance(response, base.Response):
+        if isinstance(response, zerojson.Future):
+            response.add_callback(self._after_call, iface)
+            return response
+
+        if not isinstance(response, zerojson.Response):
             response = request.response(response)
 
+        return self._after_call(iface, response)
+
+    # Private
+
+    def _after_call(self, iface, response):
+        """Routine to run after call."""
         if len(self.results) == 1:
             response.result = {self.results[0]: response.result}
         elif len(self.results) > 1:
@@ -268,7 +266,6 @@ class Method(object):
 
         return response
 
-
 # Private
 
 
@@ -277,5 +274,5 @@ def _fetch_argument(args, name, method_name, iface_name):
     try:
         return args[name]
     except KeyError:
-        raise exceptions.BadRequest("Argument '%s' is required for %s.%s" %
+        raise zerojson.BadRequest("Argument '%s' is required for %s.%s" %
             (name, method_name, iface_name))
