@@ -31,11 +31,11 @@ import datetime
 import six
 
 import pycom
-from pycom import constants, exceptions
+from pycom import constants
 from pycom.apps import nameserver
-from pycom.zerojson import client as protocol
 
-from .utils import unittest, Replacer, OrderedDict, OrderedDictWithSet
+from zerojson.testing import unittest, Replacer
+from zerojson.testing import OrderedDict, OrderedDictWithSet
 
 
 class BaseTest(object):
@@ -155,11 +155,11 @@ class TestRegister(BaseTest, unittest.TestCase):
             set())
 
     def testRegisterNone(self):
-        self.assertRaises(exceptions.BadRequest, self.component.invoke,
+        self.assertRaises(pycom.BadRequest, self.component.invoke,
             constants.NS_METHOD_REGISTER)
 
     def testRegisterWOAddress(self):
-        self.assertRaises(exceptions.BadRequest, self.component.invoke,
+        self.assertRaises(pycom.BadRequest, self.component.invoke,
             constants.NS_METHOD_REGISTER,
             args={
                 "interfaces": ["test_apps_nameserver.testRegisterWOAddress"],
@@ -167,7 +167,7 @@ class TestRegister(BaseTest, unittest.TestCase):
             })
 
     def testRegisterWrongList(self):
-        self.assertRaises(exceptions.BadRequest, self.component.invoke,
+        self.assertRaises(pycom.BadRequest, self.component.invoke,
             constants.NS_METHOD_REGISTER,
             args={
                 "interfaces": 42,
@@ -176,7 +176,7 @@ class TestRegister(BaseTest, unittest.TestCase):
             })
 
     def testRegisterWrongItem(self):
-        self.assertRaises(exceptions.BadRequest, self.component.invoke,
+        self.assertRaises(pycom.BadRequest, self.component.invoke,
             constants.NS_METHOD_REGISTER,
             args={
                 "interfaces": ["s", 42],
@@ -277,11 +277,11 @@ class TestLocate(TestWithData, unittest.TestCase):
             })
 
     def testLocateNone(self):
-        self.assertRaises(exceptions.BadRequest, self.component.invoke,
+        self.assertRaises(pycom.BadRequest, self.component.invoke,
             constants.NS_METHOD_LOCATE)
 
     def testLocateEmpty(self):
-        self.assertRaises(exceptions.BadRequest, self.component.invoke,
+        self.assertRaises(pycom.BadRequest, self.component.invoke,
             constants.NS_METHOD_LOCATE,
             args={})
 
@@ -308,7 +308,7 @@ class TestLocate(TestWithData, unittest.TestCase):
             })
 
     def testLocateMissing(self):
-        self.assertRaises(exceptions.ServiceNotFound, self.component.invoke,
+        self.assertRaises(pycom.ServiceNotFound, self.component.invoke,
             constants.NS_METHOD_LOCATE,
             args={
                 "interface": "test_apps_nameserver.testLocateMissing"
