@@ -30,9 +30,10 @@ import functools
 import re
 
 import pycom
-from pycom import base, constants, exceptions, interfaces
+import zerojson
+from pycom import base, constants, interfaces
 
-from .utils import unittest
+from zerojson.testing import unittest
 
 class BaseTest(object):
 
@@ -70,6 +71,22 @@ class DecoratorsTest(BaseTest, unittest.TestCase):
             @pycom.method(meth2)
             def m2(self): pass # pragma: no cover
 
+        self.assertIn(meth1, InterfaceWithMethods.__interface__.methods)
+        self.assertIn(meth2, InterfaceWithMethods.__interface__.methods)
+
+    def testInterfaceWithImplicitMethods(self):
+        iface = "test.test_interfaces.DecoratorsTest." \
+            "testInterfaceWithImplicitMethods"
+
+        @pycom.interface(iface)
+        class InterfaceWithMethods(object):
+
+            @pycom.method
+            def implicit_method(self): pass # pragma: no cover
+
+        self.assertIn("implicit_method",
+            InterfaceWithMethods.__interface__.methods)
+
     def testErrorOnDuplicateInterface(self):
         iface = "test.test_interfaces.DecoratorsTest.testErrorOnDuplicateInterface"
 
@@ -102,7 +119,7 @@ class DecoratorsTest(BaseTest, unittest.TestCase):
             (re.escape(meth1), re.escape(iface)), doIt)
 
 
-FakeRequest = functools.partial(pycom.Request, None, None, None)
+FakeRequest = functools.partial(zerojson.Request, None, None)
 
 
 class HooksTest(BaseTest, unittest.TestCase):
@@ -115,7 +132,7 @@ class HooksTest(BaseTest, unittest.TestCase):
             def _hook(iface, method, request):
                 self.assertIsInstance(iface, interfaces.Interface)
                 self.assertIsInstance(method, interfaces.Method)
-                self.assertIsInstance(request, base.Request)
+                self.assertIsInstance(request, zerojson.Request)
                 request.args = {"replaced": request.args}
                 return request
             meth.__method__.prehooks.append(_hook)
@@ -132,7 +149,7 @@ class HooksTest(BaseTest, unittest.TestCase):
 
         instance = InterfaceWithMethods()
         response = instance.m1.__method__.call(instance.__interface__,
-            FakeRequest(42))
+            FakeRequest(args=42))
         self.assertEqual(response.result, 100500)
 
     def testPostHook(self):
@@ -143,7 +160,7 @@ class HooksTest(BaseTest, unittest.TestCase):
             def _hook(iface, method, response):
                 self.assertIsInstance(iface, interfaces.Interface)
                 self.assertIsInstance(method, interfaces.Method)
-                self.assertIsInstance(response, base.Response)
+                self.assertIsInstance(response, zerojson.Response)
                 response.result = {"replaced": response.result}
                 return response
             meth.__method__.posthooks.append(_hook)
@@ -160,7 +177,7 @@ class HooksTest(BaseTest, unittest.TestCase):
 
         instance = InterfaceWithMethods()
         response = instance.m1.__method__.call(instance.__interface__,
-            FakeRequest(42))
+            FakeRequest(args=42))
         self.assertDictEqual(response.result, {"replaced": 100500})
 
 
@@ -185,6 +202,25 @@ class LocalComponentTest(BaseTest, unittest.TestCase):
         self.assertIsInstance(component, pycom.BaseComponent)
         self.assertIs(component.invoke(meth1, args=testObject1), testObject2)
 
+    def testCallImplicit(self):
+        iface = "test.test_components.LocalComponentTest.testCallImplicit"
+        testObject1 = object()
+        testObject2 = object()
+
+        @pycom.interface(iface)
+        class InterfaceWithMethods(object):
+
+            @pycom.method
+            def method1(self, request):
+                assert self.__interface__.name == iface
+                assert request.args is testObject1
+                return testObject2
+
+        component = InterfaceWithMethods().__interface__
+        self.assertIsInstance(component, pycom.BaseComponent)
+        self.assertIs(component.invoke("method1", args=testObject1),
+            testObject2)
+
     def testCallWithResponse(self):
         iface = "test.test_components.LocalComponentTest.testCallWithResponse"
         meth1 = "method1"
@@ -213,12 +249,12 @@ class LocalComponentTest(BaseTest, unittest.TestCase):
         @pycom.interface(iface)
         class InterfaceWithMethods(object): pass
 
-        def meth1(self, request):
+        def meth1_impl(self, request):
             assert request.args is testObject1
             return request.response(testObject2)
 
         InterfaceWithMethods.__interface__.register_method(
-            pycom.method(meth1, body=meth1))
+            pycom.method(meth1, body=meth1_impl))
 
         component = InterfaceWithMethods().__interface__
         self.assertIsInstance(component, pycom.BaseComponent)
@@ -236,7 +272,7 @@ class LocalComponentTest(BaseTest, unittest.TestCase):
 
         component = InterfaceWithMethods().__interface__
         self.assertIsInstance(component, pycom.BaseComponent)
-        self.assertRaises(exceptions.MethodNotFound,
+        self.assertRaises(pycom.MethodNotFound,
             component.invoke, "banana!")
 
     def testCallWrongName(self):
@@ -251,7 +287,7 @@ class LocalComponentTest(BaseTest, unittest.TestCase):
 
         component = InterfaceWithMethods().__interface__
         self.assertIsInstance(component, pycom.BaseComponent)
-        self.assertRaises(exceptions.MethodNotFound, component.invoke, "m1")
+        self.assertRaises(pycom.MethodNotFound, component.invoke, "m1")
 
 
 class ArgumentsTest(BaseTest, unittest.TestCase):
@@ -270,11 +306,11 @@ class ArgumentsTest(BaseTest, unittest.TestCase):
         component = SimpleInterface.__interface__
         self.assertEqual(component.invoke(meth, {"arg1": 1, "arg2": 2}),
             [1, 2])
-        self.assertRaisesRegexp(exceptions.BadRequest,
+        self.assertRaisesRegexp(pycom.BadRequest,
             "test.test_interfaces.ArgumentsTest.testRequiredOnly.method1",
             component.invoke, meth,
             {"arg2": 2})
-        self.assertRaisesRegexp(exceptions.BadRequest,
+        self.assertRaisesRegexp(pycom.BadRequest,
             "test.test_interfaces.ArgumentsTest.testRequiredOnly.method1",
             component.invoke, meth,
             42)
@@ -295,7 +331,7 @@ class ArgumentsTest(BaseTest, unittest.TestCase):
             [1, 2])
         self.assertEqual(component.invoke(meth, {"arg2": 2}),
             [100500, 2])
-        self.assertRaisesRegexp(exceptions.BadRequest,
+        self.assertRaisesRegexp(pycom.BadRequest,
             "test.test_interfaces.ArgumentsTest.testOptionalOnly.method1",
             component.invoke, meth,
             42)
@@ -332,6 +368,20 @@ class ArgumentsTest(BaseTest, unittest.TestCase):
         self.assertEqual(component.invoke(meth),
             {"res1": 0})
 
+    def testImplicitWResults(self):
+        iface = "test.test_components.ArgumentsTest.testImplicitWResults"
+
+        @pycom.interface(iface)
+        class SimpleInterface(object):
+
+            @pycom.method(results=("res1",))
+            def method1(self, request):
+                return 0
+
+        component = SimpleInterface.__interface__
+        self.assertEqual(component.invoke("method1"),
+            {"res1": 0})
+
     def testBadInterface(self):
         iface = "test.test_interfaces.ArgumentsTest.testBadInterface"
         meth = "method1"
@@ -377,11 +427,11 @@ class ArgumentsTest(BaseTest, unittest.TestCase):
             [0, 1, 2])
         self.assertEqual(component.invoke(meth, {"arg0": 0}),
             [0, 100500, 42])
-        self.assertRaisesRegexp(exceptions.BadRequest,
+        self.assertRaisesRegexp(pycom.BadRequest,
             "test.test_interfaces.ArgumentsTest.testAllArguments.method1",
             component.invoke, meth,
             {"arg2": 2})
-        self.assertRaisesRegexp(exceptions.BadRequest,
+        self.assertRaisesRegexp(pycom.BadRequest,
             "test.test_interfaces.ArgumentsTest.testAllArguments.method1",
             component.invoke, meth,
             42)
