@@ -39,19 +39,21 @@ from six import StringIO
 
 import pycom
 import pycom.apps.nameserver
+import zerojson
 from pycom import base, constants, nsclient, utils
-from pycom.zerojson import call, client as protocol
+from zerojson import call
 
-from .utils import unittest, Replacer
+from zerojson.testing import unittest, AddressGenerationMixin, Replacer
 from .test_nsclient import BaseTest
 
 
-class MainTest(BaseTest, unittest.TestCase):
+class MainTest(BaseTest, AddressGenerationMixin, unittest.TestCase):
 
     timeout = 3
 
     def setUp(self):
         super(MainTest, self).setUp()
+        AddressGenerationMixin.setUp(self)
         self.oldConf = base.configuration.copy()
         base.configuration.clear()
 
@@ -67,6 +69,7 @@ class MainTest(BaseTest, unittest.TestCase):
         self.context = None
         base.configuration.clear()
         base.configuration.update(self.oldConf)
+        AddressGenerationMixin.tearDown(self)
         super(MainTest, self).tearDown()
 
     def testService(self):
@@ -74,12 +77,32 @@ class MainTest(BaseTest, unittest.TestCase):
         meth = "test_method"
 
         def _check(address, self):
-            component = protocol.RemoteComponent(address, iface)
+            component = nsclient.RemoteComponent(address, iface)
             self.assertDictEqual(component.invoke(meth, [1, 2, 3]),
                 {"input": [1, 2, 3]})
 
         self._testService(iface, meth, _testMethod, _check, self)
 
+    def testFuture(self):
+        iface = "test.test_launcher.MainTest.testFuture"
+        meth = "test_method"
+
+        def _finish(future):
+            future.response({"input": future.request.args})
+
+        def _method(request):
+            self.assertEqual(request.interface, iface)
+            future = zerojson.Future(request)
+            pycom.ioloop().add_callback(functools.partial(_finish, future))
+            return future
+
+        def _check(address, self):
+            component = nsclient.RemoteComponent(address, iface)
+            self.assertDictEqual(component.invoke(meth, [1, 2, 3]),
+                {"input": [1, 2, 3]})
+
+        self._testService(iface, meth, _method, _check, self)
+
     def testConfigurationError(self):
         pycom.configure(service="/test/test_launcher")
         self.assertRaisesRegexp(pycom.ConfigurationError, "address",
@@ -91,149 +114,12 @@ class MainTest(BaseTest, unittest.TestCase):
 
         def _check(address, self):
             wrong_iface = iface + "/banana!"
-            component = protocol.RemoteComponent(address, wrong_iface)
+            component = nsclient.RemoteComponent(address, wrong_iface)
             with self.assertRaisesRegexp(pycom.RemoteError,
                     "Unsupported interface: %s" % wrong_iface) as context:
                 component.invoke(meth)
             self.assertEqual(context.exception.code,
-                constants.ERROR_BAD_REQUEST)
-
-        self._testService(iface, meth, _testMethod, _check, self)
-
-    def testUnknownMethod(self):
-        iface = "test.test_launcher.MainTest.testUnknownMethod"
-        meth = "test_method"
-
-        def _check(address, self):
-            wrong_meth = meth + "/banana!"
-            component = protocol.RemoteComponent(address, iface)
-            with self.assertRaisesRegexp(pycom.MethodNotFound,
-                    "Method '%s' not found in interface %s" %
-                    (wrong_meth, iface)) as context:
-                component.invoke(wrong_meth)
-            self.assertEqual(context.exception.code,
-                constants.ERROR_METHOD_NOT_FOUND)
-
-        self._testService(iface, meth, _testMethod, _check, self)
-
-    def testKnownException(self):
-        iface = "test.test_launcher.MainTest.testKnownException"
-        meth = "test_method"
-
-        def _check(address, self):
-            component = protocol.RemoteComponent(address, iface)
-            with self.assertRaisesRegexp(pycom.RemoteError,
-                    "test BadRequest") as context:
-                component.invoke(meth)
-            self.assertEqual(context.exception.code,
-                constants.ERROR_BAD_REQUEST)
-
-        self._testService(iface, meth, _testMethodWithKnownError, _check, self)
-
-    def testUnknownException(self):
-        iface = "test.test_launcher.MainTest.testUnknownException"
-        meth = "test_method"
-
-        def _check(address, self):
-            component = protocol.RemoteComponent(address, iface)
-            with self.assertRaisesRegexp(pycom.RemoteError,
-                    "Unknown error during request") as context:
-                component.invoke(meth)
-            self.assertEqual(context.exception.code,
-                constants.ERROR_UNKNOWN)
-
-        self._testService(iface, meth, _testMethodWithUnknownError, _check, self)
-
-    def testSession(self):
-        iface = "test.test_launcher.MainTest.testSession"
-        meth = "test_method"
-
-        def _check(address, self):
-            component = protocol.RemoteComponent(address, iface)
-            self.assertEqual(component.invoke(meth, 42), 42)
-            self.assertEqual(component.invoke(meth, 100500), 100500)
-            self.assertEqual(component.invoke(meth), 100500)
-
-        self._testService(iface, meth, _testSession, _check, self)
-
-    def testUnknownCommand(self):
-        iface = "test.test_launcher.MainTest.testUnknownCommand"
-        meth = "test_method"
-
-        def _check(address, self):
-            component = protocol.RemoteComponent(address, iface)
-            with self.assertRaisesRegexp(pycom.RemoteError,
-                    "Unknown command") as context:
-                message = component._send_message(b"banana!", b"{}", 1000)[1]
-                self.assertTrue(message)
-                call.CallCommand().response_from_wire(message)
-            self.assertEqual(context.exception.code,
-                constants.ERROR_BAD_REQUEST)
-
-        self._testService(iface, meth, _testMethod, _check, self)
-
-    def testWrongJSON(self):
-        iface = "test.test_launcher.MainTest.testWrongJSON"
-        meth = "test_method"
-
-        def _check(address, self):
-            component = protocol.RemoteComponent(address, iface)
-            with self.assertRaisesRegexp(pycom.RemoteError,
-                    "Cannot decode response") as context:
-                message = component._send_message(constants.PROTO_CMD_CALL,
-                    b"!!!", 1000)[1]
-                call.CallCommand().response_from_wire(message)
-            self.assertEqual(context.exception.code,
-                constants.ERROR_BAD_REQUEST)
-
-        self._testService(iface, meth, _testMethod, _check, self)
-
-    def testMissingFields(self):
-        iface = "test.test_launcher.MainTest.testMissingFields"
-        meth = "test_method"
-
-        def _check(address, self):
-            component = protocol.RemoteComponent(address, iface)
-            with self.assertRaisesRegexp(pycom.RemoteError,
-                    "Field [a-z]+ is required") as context:
-                message = component._send_message(constants.PROTO_CMD_CALL,
-                    b"{}", 1000)[1]
-                call.CallCommand().response_from_wire(message)
-            self.assertEqual(context.exception.code,
-                constants.ERROR_BAD_REQUEST)
-
-        self._testService(iface, meth, _testMethod, _check, self)
-
-    def testWrongUTF8(self):
-        iface = "test.test_launcher.MainTest.testWrongUTF8"
-        meth = "test_method"
-
-        def _check(address, self):
-            component = protocol.RemoteComponent(address, iface)
-            with self.assertRaisesRegexp(pycom.RemoteError,
-                    "Invalid encoding, expected UTF-8") as context:
-                message = component._send_message(constants.PROTO_CMD_CALL,
-                    b"{\"v1\": \"\xff\xff\"}", 1000)[1]
-                call.CallCommand().response_from_wire(message)
-            self.assertEqual(context.exception.code,
-                constants.ERROR_BAD_REQUEST)
-
-        self._testService(iface, meth, _testMethod, _check, self)
-
-    def testWrongPartsCount(self):
-        iface = "test.test_launcher.MainTest.testWrongPartsCount"
-        meth = "test_method"
-
-        def _check(address, self):
-            component = protocol.RemoteComponent(address, iface)
-            with self.assertRaisesRegexp(pycom.RemoteError,
-                    "Expected two parts, got 3") as context:
-                message = component._send_message(constants.PROTO_CMD_CALL,
-                    call.CallCommand().request_to_wire(iface, meth, {}),
-                    1000, attachment=b"")[1]
-                call.CallCommand().response_from_wire(message)
-            self.assertEqual(context.exception.code,
-                constants.ERROR_BAD_REQUEST)
+                zerojson.constants.ERROR_BAD_REQUEST)
 
         self._testService(iface, meth, _testMethod, _check, self)
 
@@ -248,7 +134,7 @@ class MainTest(BaseTest, unittest.TestCase):
             def test_method(self, request):
                 return request.args
 
-        address = "ipc:///tmp/pycom_test_launcher-%d" % uuid.uuid4().int
+        address = self.genAddress()
 
         def _check():
             try:
@@ -308,7 +194,7 @@ class MainTest(BaseTest, unittest.TestCase):
             def test_method(self, request):
                 return meth_body(request)
 
-        address = "ipc:///tmp/pycom_test_launcher-%d" % uuid.uuid4().int
+        address = self.genAddress()
 
         def _check():
             try:
@@ -324,25 +210,13 @@ class MainTest(BaseTest, unittest.TestCase):
 
 _testMethod = lambda request: {"input": request.args}
 
-def _testMethodWithKnownError(request):
-    raise pycom.BadRequest("test BadRequest")
-
-def _testMethodWithUnknownError(request):
-    raise TypeError("test TypeError")
-
-def _testSession(request):
-    if request.args:
-        request.session["test"] = request.args
-
-    return request.session["test"]
-
 
 class DummyNS(object):
 
     def invoke(*args, **kwargs): pass
 
 
-class ArgsTest(unittest.TestCase):
+class ArgsTest(AddressGenerationMixin, unittest.TestCase):
 
     def setUp(self):
         super(ArgsTest, self).setUp()
@@ -368,7 +242,7 @@ class ArgsTest(unittest.TestCase):
             self.assertRaises(SystemExit, self.main, ["sys", "-a"])
 
     def testGeneratedService(self):
-        address = "ipc:///tmp/pycom_test_launcher-%d" % uuid.uuid4().int
+        address = self.genAddress()
 
         pycom.ioloop().add_callback(pycom.ioloop().stop)
         self.main(["sys", "-a", address])
@@ -378,13 +252,13 @@ class ArgsTest(unittest.TestCase):
             self.assertRaises(SystemExit, self.main, ["sys", "-x"])
 
     def testCorrectOptions(self):
-        address = "ipc:///tmp/pycom_test_launcher-%d" % uuid.uuid4().int
+        address = self.genAddress()
 
         pycom.ioloop().add_callback(pycom.ioloop().stop)
         self.main(["sys", "-a", address, "-s", "/test/test_launcher2"])
 
     def testNameserverOption(self):
-        address = "ipc:///tmp/pycom_test_launcher-%d" % uuid.uuid4().int
+        address = self.genAddress()
 
         def _check():
             try:
@@ -398,7 +272,7 @@ class ArgsTest(unittest.TestCase):
             "-n", "test_ns_vi_cmd"])
 
     def testUnknownModule(self):
-        address = "ipc:///tmp/pycom_test_launcher-%d" % uuid.uuid4().int
+        address = self.genAddress()
 
         with Replacer(sys, "stderr", StringIO()):
             self.assertRaises(SystemExit, self.main,
