@@ -32,50 +32,52 @@ import threading
 import zmq
 
 import pycom
-from pycom import constants, exceptions, interfaces, nsclient
-from pycom.zerojson import call, common, client as protocol
+import zerojson
+from pycom import base, constants, interfaces, nsclient
+from zerojson import call, common
 
-from .utils import unittest, Replacer
+from zerojson.testing import unittest, Replacer
 from .test_interfaces import BaseTest as InterfacesBaseTest
 
+
 class BaseTest(InterfacesBaseTest):
 
     timeout = 3
 
     def setUp(self):
         super(BaseTest, self).setUp()
-        self.oldNS = nsclient._pop_cached_ns()
         pycom.configure(nameserver="local")
 
     def tearDown(self):
-        nsclient._restore_cached_ns(self.oldNS)
         super(BaseTest, self).tearDown()
 
 class LocalNSTest(BaseTest, unittest.TestCase):
 
     def testErrorOnAbsentNameserver(self):
-        self.assertRaises(exceptions.ServiceNotFound,
+        self.assertRaises(zerojson.ServiceNotFound,
             pycom.Context().nameserver)
 
     def testErrorOnMalformedNS(self):
         pycom.configure(nameserver="host:unknown.unknown")
 
-        @pycom.interface(constants.IFACE_NAMESERVER)
+        @pycom.interface(constants.NS_INTERFACE)
         class Nameserver(object): pass
 
-        self.assertRaises(exceptions.ConfigurationError,
+        self.assertRaises(base.ConfigurationError,
             pycom.Context().nameserver)
 
     def testNsInThisService(self):
-        @pycom.interface(constants.IFACE_NAMESERVER)
+        @pycom.interface(constants.NS_INTERFACE)
         class Nameserver(object): pass
 
-        ns = pycom.Context().nameserver()
+        ctx = pycom.Context()
+        ns = ctx.nameserver()
         self.assertIsInstance(ns, pycom.BaseComponent)
         self.assertIsInstance(ns.instance, Nameserver)
+        self.assertIs(ns.context, ctx)
 
     def testCachedNs(self):
-        @pycom.interface(constants.IFACE_NAMESERVER)
+        @pycom.interface(constants.NS_INTERFACE)
         class Nameserver(object): pass
 
         context = pycom.Context()
@@ -99,25 +101,25 @@ def fake_container(context, event, addr, event2, handler):
             continue
 
         cmd, body = sock.recv_multipart()
-        assert cmd == constants.PROTO_CMD_CALL
+        assert cmd == zerojson.constants.PROTO_CMD_CALL
 
         request = call_command.request_from_wire(body)
-        sock.send_multipart([constants.PROTO_STATUS_SUCCESS,
+        sock.send_multipart([zerojson.constants.PROTO_STATUS_SUCCESS,
             call_command.response_to_wire(handler(request))])
 
     sock.close()
 
 
 def ns_handler(request):
-    assert request.interface == constants.IFACE_NAMESERVER
+    assert request.interface == constants.NS_INTERFACE
     assert request.method in (constants.NS_METHOD_LOCATE,
         constants.NS_METHOD_STAT)
 
     if request.method == constants.NS_METHOD_STAT:
         return request.response({})
     else:
-        return pycom.Response(request,
-                code=constants.ERROR_SERVICE_NOT_FOUND)
+        return zerojson.Response(request,
+                code=zerojson.constants.ERROR_SERVICE_NOT_FOUND)
 
 
 class RemoteNSTest(BaseTest, unittest.TestCase):
@@ -148,7 +150,7 @@ class RemoteNSTest(BaseTest, unittest.TestCase):
     def testNS(self):
         ns = pycom.Context().nameserver()
         self.assertDictEqual(ns.invoke(constants.NS_METHOD_STAT), {})
-        self.assertRaises(exceptions.RemoteError,
+        self.assertRaises(pycom.RemoteError,
             ns.invoke, constants.NS_METHOD_LOCATE, {
                 "interface": "org.pycom.test_nsclient.NON-EXISTING"
             })
@@ -156,23 +158,23 @@ class RemoteNSTest(BaseTest, unittest.TestCase):
     def testNSContextManager(self):
         with pycom.Context().nameserver() as ns:
             self.assertDictEqual(ns.invoke(constants.NS_METHOD_STAT), {})
-            self.assertRaises(exceptions.RemoteError,
+            self.assertRaises(pycom.RemoteError,
                 ns.invoke, constants.NS_METHOD_LOCATE, {
                     "interface": "org.pycom.test_nsclient.NON-EXISTING"
                 })
 
     def testUnknownNS(self):
         context = pycom.Context(nameserver="ipc://pycom/banana!")
-        with Replacer(constants, "PROTO_DEFAULT_TIMEOUT", 100):
-            self.assertRaises(exceptions.ServiceNotFound, context.nameserver)
+        with Replacer(zerojson.constants, "PROTO_DEFAULT_TIMEOUT", 100):
+            self.assertRaises(zerojson.ServiceNotFound, context.nameserver)
 
     def testUnknownNS2(self):
         context = pycom.Context(nameserver="inproc://pycom/banana!")
-        with Replacer(constants, "PROTO_DEFAULT_TIMEOUT", 100):
-            self.assertRaises(exceptions.ServiceNotFound, context.nameserver)
+        with Replacer(zerojson.constants, "PROTO_DEFAULT_TIMEOUT", 100):
+            self.assertRaises(zerojson.ServiceNotFound, context.nameserver)
 
     def testLocateError(self):
-        self.assertRaises(exceptions.ServiceNotFound,
+        self.assertRaises(zerojson.ServiceNotFound,
             pycom.Context().locate, "org.pycom.test_nsclient.NON-EXISTING")
 
 
@@ -205,26 +207,26 @@ class RemoteNSTestContext(BaseTest, unittest.TestCase):
         self.context = pycom.Context(nameserver=self.addr)
         ns = self.context.nameserver()
         self.assertDictEqual(ns.invoke(constants.NS_METHOD_STAT), {})
-        self.assertRaises(exceptions.RemoteError,
+        self.assertRaises(pycom.RemoteError,
             ns.invoke, constants.NS_METHOD_LOCATE, {
                 "interface": "org.pycom.test_nsclient.NON-EXISTING"
             })
 
     def testLocate(self):
         self.context = pycom.Context(nameserver=self.addr)
-        self.assertRaises(exceptions.ServiceNotFound,
+        self.assertRaises(zerojson.ServiceNotFound,
             self.context.locate, "org.pycom.test_nsclient.NON-EXISTING")
 
 
 def ns_handler_w_error(request):
-    assert request.interface == constants.IFACE_NAMESERVER
+    assert request.interface == constants.NS_INTERFACE
     assert request.method in (constants.NS_METHOD_LOCATE,
         constants.NS_METHOD_STAT)
 
     if request.method == constants.NS_METHOD_STAT:
         return request.response({})
     else:
-        return pycom.Response(request, code=100500)
+        return zerojson.Response(request, code=100500)
 
 
 class LocateFailureTest(BaseTest, unittest.TestCase):
@@ -253,13 +255,13 @@ class LocateFailureTest(BaseTest, unittest.TestCase):
         super(LocateFailureTest, self).tearDown()
 
     def testUnknownError(self):
-        with self.assertRaises(exceptions.RemoteError) as cm:
+        with self.assertRaises(pycom.RemoteError) as cm:
             pycom.Context().locate("org.pycom.test_nsclient.NON-EXISTING")
         self.assertEqual(cm.exception.code, 100500)
 
 
 def ns_handler2(request):
-    assert request.interface == constants.IFACE_NAMESERVER
+    assert request.interface == constants.NS_INTERFACE
     assert request.method in (constants.NS_METHOD_LOCATE,
         constants.NS_METHOD_STAT)
 
@@ -273,8 +275,8 @@ def ns_handler2(request):
             assert request.args["service"] == "service.name2"
             return request.response({"address": "ipc://test_nsclient_h2"})
         else:
-            return pycom.Response(request,
-                code=constants.ERROR_SERVICE_NOT_FOUND)
+            return zerojson.Response(request,
+                code=zerojson.constants.ERROR_SERVICE_NOT_FOUND)
 
 
 class LocateTest(BaseTest, unittest.TestCase):
@@ -304,7 +306,7 @@ class LocateTest(BaseTest, unittest.TestCase):
 
     def testConnect(self):
         ctx = pycom.Context()
-        component = ctx.connect(self.addr, constants.IFACE_NAMESERVER)
+        component = ctx.connect(self.addr, constants.NS_INTERFACE)
         self.assertIsInstance(component, pycom.BaseComponent)
         self.assertEqual(component.invoke(constants.NS_METHOD_STAT), {})
         self.assertIs(component.context, ctx)
@@ -313,23 +315,60 @@ class LocateTest(BaseTest, unittest.TestCase):
         ctx = pycom.Context()
         component = ctx.locate("iface.name")
         self.assertIsInstance(component, pycom.BaseComponent)
-        self.assertEqual(component.iface, "iface.name")
-        self.assertIsInstance(component.socket, zmq.Socket)
+        self.assertEqual(component.name, "iface.name")
+        self.assertIsInstance(component._wrapped, zerojson.Client)
         self.assertIs(component.context, ctx)
 
     def testLocateContextManager(self):
         with pycom.Context().locate("iface.name") as component:
             self.assertIsInstance(component, pycom.BaseComponent)
-            self.assertEqual(component.iface, "iface.name")
-            self.assertIsInstance(component.socket, zmq.Socket)
+            self.assertEqual(component.name, "iface.name")
+            self.assertIsInstance(component._wrapped, zerojson.Client)
 
     def testLocateWService(self):
         component = pycom.Context().locate("iface.name2",
             service_name="service.name2")
         self.assertIsInstance(component, pycom.BaseComponent)
-        self.assertEqual(component.iface, "iface.name2")
-        self.assertIsInstance(component.socket, zmq.Socket)
+        self.assertEqual(component.name, "iface.name2")
+        self.assertIsInstance(component._wrapped, zerojson.Client)
 
     def testLocateFail(self):
-        self.assertRaises(exceptions.ServiceNotFound,
+        self.assertRaises(zerojson.ServiceNotFound,
             pycom.Context().locate, "org.pycom.test_nsclient.NON-EXISTING")
+
+
+class ContextHooksTest(BaseTest, unittest.TestCase):
+
+    def setUp(self):
+        super(ContextHooksTest, self).setUp()
+
+        @pycom.interface(constants.NS_INTERFACE)
+        class Nameserver(object):
+
+            @pycom.method
+            def stat(self2, request):
+                return request.args
+
+    def testPrehook(self):
+        def _hook(comp, request):
+            self.assertIsInstance(comp, base.BaseComponent)
+            self.assertIsInstance(request, zerojson.Request)
+            request.args = {"input": request.args}
+            return request
+
+        ctx = pycom.Context()
+        ctx.prehooks.append(_hook)
+        self.assertEqual(ctx.nameserver().invoke("stat", args=42),
+            {"input": 42})
+
+    def testPosthook(self):
+        def _hook(comp, response):
+            self.assertIsInstance(comp, base.BaseComponent)
+            self.assertIsInstance(response, zerojson.Response)
+            response.result = {"output": response.result}
+            return response
+
+        ctx = pycom.Context()
+        ctx.posthooks.append(_hook)
+        self.assertEqual(ctx.nameserver().invoke("stat", args=42),
+            {"output": 42})
